<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Method;

use TecDiet\Services\PatientsService;

class PatientsController extends \Zeedhi\Framework\Controller\Simple {

    protected $patientsService;

    public function __construct(PatientsService $patientsService) {
        $this->patientsService = $patientsService;
    }

    public function save(Request\DataSet $request, Response $response) {
        $patients = $request->getDataSet()->getRows();
        if($patients) {

            try {

                $dataSet = $this->patientsService->save($patients);
                if(sizeof($dataSet) != 0){
                    $response->addDataSet(new DataSet('error', $dataSet));
                }

            } catch(\Exception $e) {
                $response->setError(new Error($e->getMessage(), $e->getCode()));
            };

        }

    }
    
    public function updateFoodRestriction(Request\DataSet $request, Response $response) {
        $rows = $request->getDataSet()->getRows();
        $patients = $this->patientsService->updateFoodRestriction($rows, $response);
        if(isset($patients['normalFeedings'])){
            $response->addDataSet(new DataSet ('NormalFeedings', $patients['normalFeedings']));
        }
    }

    public function patientSequentialAjusta(Request\Row $request, Response $response){
        $units = $request->getRow();
        $patientSequentialAjusta = $this->patientsService->patientSequentialAjusta($units['unit']);
        $response->addDataSet(new DataSet ('patientSequentialAjusta', $patientSequentialAjusta));
    }
    
    public function patientSequential(Request\Row $request, Response $response){
        $row = $request->getRow();
        $patientSequential = $this->patientsService->getPatientSequential($row);
        $response->addDataSet(new DataSet ('patientSequential', $patientSequential));
        
    }

    public function get(Request\Row $request, Response $response) {
        try {
            
            $units = $request->getRow();

            $patients = $this->patientsService->get($units['unitID'], $units['nutritionistID']);
            if(isset($patients['init'])) {
                $response->addDataSet(new DataSet ('Patients', $patients['init']));
            } else {
                $response->addDataSet(new DataSet('Patients',[]));
            }

            if(isset($patients['profiles'])) {
                $response->addDataSet(new DataSet ('Profiles', $patients['profiles']));
            } else {
                $response->addDataSet(new DataSet('Profiles',[]));
            }

            if(isset($patients['prescriptions'])) {
                $response->addDataSet(new DataSet ('Prescriptions', $patients['prescriptions']));
            } else {
                $response->addDataSet(new DataSet('Prescriptions',[]));
            }
            
            if(isset($patients['appointments'])) {
                $response->addDataSet(new DataSet ('Appointments', $patients['appointments']));
            } else {
                $response->addDataSet(new DataSet('Appointments',[]));
            }

            if(isset($patients['anthEvaluationMethods'])) {
                $response->addDataSet(new DataSet ('AnthEvaluationMethods', $patients['anthEvaluationMethods']));
            } else {
                $response->addDataSet(new DataSet('AnthEvaluationMethods',[]));
            }

            if(isset($patients['anthEvaluationGeneral'])) {
                $response->addDataSet(new DataSet ('AnthEvaluationGeneral', $patients['anthEvaluationGeneral']));
            } else {
                $response->addDataSet(new DataSet('AnthEvaluationGeneral',[]));
            }

            if(isset($patients['controlFeedings'])) {
                $response->addDataSet(new DataSet ('ControlFeedings', $patients['controlFeedings']));
            } else {
                $response->addDataSet(new DataSet('ControlFeedings',[]));
            }

            if(isset($patients['enteralFeedings'])) {
                $response->addDataSet(new DataSet ('EnteralFeedings', $patients['enteralFeedings']));
            } else {
                $response->addDataSet(new DataSet('EnteralFeedings',[]));
            }

            if(isset($patients['lactaryFeedings'])) {
                $response->addDataSet(new DataSet ('LactaryFeedings', $patients['lactaryFeedings']));
            } else {
                $response->addDataSet(new DataSet('LactaryFeedings',[]));
            }

            if(isset($patients['normalFeedings'])) {
                $response->addDataSet(new DataSet ('NormalFeedings', $patients['normalFeedings']));
            } else {
                $response->addDataSet(new DataSet('NormalFeedings',[]));
            }
            
            if(isset($patients['restrictionAndIntolerance'])) {
                $response->addDataSet(new DataSet ('PatientIntoRestriction', $patients['restrictionAndIntolerance']));
            } else {
                $response->addDataSet(new DataSet('PatientIntoRestriction',[]));
            }

            if(isset($patients['bioEvaluations'])) {
                $response->addDataSet(new DataSet ('BioEvaluations', $patients['bioEvaluations']));
            } else {
                $response->addDataSet(new DataSet('BioEvaluations',[]));
            }

            if(isset($patients['dataInfoEvaluations'])) {
                $response->addDataSet(new DataSet ('DataInfoEvaluations', $patients['dataInfoEvaluations']));
            } else {
                $response->addDataSet(new DataSet('DataInfoEvaluations',[]));
            }
            if(isset($patients['anthAvaliationResults'])) {
                $response->addDataSet(new DataSet ('AnthAvaliationResults', $patients['anthAvaliationResults']));
            } else {
                $response->addDataSet(new DataSet('AnthAvaliationResults',[]));
            }

        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        };

    }
}