<?php

namespace TecDiet\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;

class Auth {

    const SECRETKEY = "tecdiet";

    protected $dataSourceManager;
    protected $authTable = "operador";

    public function __construct($dataSourceManager) {
        $this->dataSourceManager = $dataSourceManager;
    }

    public function find($filterCriteria) {
        return $this->dataSourceManager->findBy($filterCriteria);
    }

    public function buildFilterCondition($name, $operator, $value) {
        return array(
            "columnName" => $name,
            "operator" => $operator,
            "value" => $value
        );
    }

    public function authenticate($email, $password) {
	    $user = $this->getUserByEmail($email);
	    if (!$user || !$this->validatePassword($user, $password)) {
    		throw new \Exception("invalidUser");
    	}
        return $user;
    }

    public function getUserByEmail($email) {
        $conditions = array();
        $conditions[]= $this->buildFilterCondition("DSEMAILOPER", "=", $email);
        $filterCriteria = new FilterCriteria($this->authTable, $conditions);
        $dataSet = $this->find($filterCriteria);
    	return isset($dataSet->getRows()[0]) ? $dataSet->getRows()[0] : null;
    }

    public function validatePassword($user, $password) {
    	return ($user["CDSENHAOPERWEB"] == md5($password));
    }

    public function validateToken($token, $sessionName) {
        try {
            $sections = explode(".", $token);
            $content = $sections[0].".".$sections[1];
            $signature = $sections[2];
            if ($this->sign($content) != $signature) {
                throw new \Exception();
            }
            return true;
        } catch(\Exception $e) {
            $this->unsetSessionCookie($sessionName);
            return false;
        }
    }

    private function sign($data) {
        return $this->base64_url_encode(hash_hmac("sha256", $data, self::SECRETKEY, true));
    }

    public function generateToken($payload) {
        $header = [
            "typ" => "JWT"
        ];
        $unsignedToken = $this->base64_url_encode(json_encode((object)$header)).".".$this->base64_url_encode(json_encode((object)$payload));
        return $unsignedToken.".".$this->sign($unsignedToken);
    }

    static function base64_url_encode($val) {
        return strtr(base64_encode($val), '+/=', '-_,');
    }

    static function base64_url_decode($val) {
        return base64_decode(strtr($val, '-_,', '+/='));
    }

    public function unsetSessionCookie($name) {
        $this->setSessionCookie($name);
    }

    public function setSessionCookie($name, $value = "", $time = 0, $path = "/", $domain = "", $secure = false, $httponly = false) {
        setcookie($name, $value, $time, $path, $domain, "SameSite=None; Secure");
    }
}