<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use TecDiet\Util\Validation;

class BioEvaluationsService implements Queries {

    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager) {

        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();

    }

    private function insertBioEvaluation($bioEvaluation) {

        $sth = $this->connection->prepare(self::INSERT_BIOCHEMICAL_EVALUATION);
        $bioEvaluation['examinationValue'] = Validation::sanitazeNumber($bioEvaluation['examinationValue'], false); 

        $envio = array(
            'DSRECLASSI' => $bioEvaluation['examinationStatus'],
            'VRRESEXA' => $bioEvaluation['examinationValue'],
            'NRSEQEXAMREA' => $bioEvaluation['priority'],
            'CDPACIENTE' => $bioEvaluation['id'],
            'DTCONSPACI' => $bioEvaluation['appointmentDate'],
            'CDEXAME' => $bioEvaluation['examinationId'],
            'DTREALEX' => $bioEvaluation['examinationDate'],
            'CDFILIAL' => $bioEvaluation['unit'],
            'IDGRAVIDEZ' => $bioEvaluation['pregnancyId'],
            'NRSEMGEST' => $bioEvaluation['pregnancyWeek']
        );

        $sth->execute($envio);

    }

    private function updateBioEvaluation($bioEvaluation) {

        $sth = $this->connection->prepare(self::UPDATE_BIOCHEMICAL_EVALUATION);

        $bioEvaluation['examinationValue'] = Validation::sanitazeNumber($bioEvaluation['examinationValue'], false); 

        $sth->bindParam(':DSRECLASSI', $bioEvaluation['examinationStatus']);
        $sth->bindParam(':VRRESEXA', $bioEvaluation['examinationValue']);
        $sth->bindParam(':CDEXAME', $bioEvaluation['examinationId']);
        $sth->bindParam(':NRSEQEXAMREA', $bioEvaluation['priority']);
        $sth->bindParam(':OLD_CDPACIENTE', $bioEvaluation['id']);
        $sth->bindParam(':OLD_DTCONSPACI', $bioEvaluation['appointmentDate']);
        $sth->bindParam(':OLD_CDEXAME', $bioEvaluation['oldExaminationId']);
        $sth->bindParam(':OLD_DTREALEX', $bioEvaluation['examinationDate']);
        $sth->bindParam(':OLD_CDFILIAL', $bioEvaluation['unit']);
        $sth->bindParam(':IDGRAVIDEZ', $bioEvaluation['pregnancyId']);
        $sth->bindParam(':NRSEMGEST', $bioEvaluation['pregnancyWeek']);

        $sth->execute();

    }

    public function save($bioEvaluations) {
        $erroToSaveBioEvaluation = array(
            'BIOEVALUATION' => array(),
            'ERROR' => array(),
            'OPERATION' => array()
        );
        foreach ($bioEvaluations as $bioEvaluation) {
            if($bioEvaluation['__is_new'] == true) {
                try {
                    $this->insertBioEvaluation($bioEvaluation);
                } catch (\Exception $e) {
                    array_push($erroToSaveBioEvaluation['BIOEVALUATION'],$bioEvaluation);
                    array_push($erroToSaveBioEvaluation['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveBioEvaluation['OPERATION'],'INSERT');
                }
            } else {
                try {
                    $this->updateBioEvaluation($bioEvaluation);
                } catch (\Exception $e) {
                    array_push($erroToSaveBioEvaluation['BIOEVALUATION'],$bioEvaluation);
                    array_push($erroToSaveBioEvaluation['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveBioEvaluation['OPERATION'],'UPDATE');
                }
            }

        }
        return $erroToSaveBioEvaluation;
    }

}