<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use TecDiet\Util\Validation;
use TecDiet\Util\CRUD;
use TecDiet\Services\NovoCodigo;

class NormalFeedingsService implements Queries {

    const SELECTED = 'S';
    const UNSELECTED = 'N';

    protected $entityManager;
    protected $connection;
    protected $crud;
    protected $novoCodigo;
    
    public function __construct(EntityManager $entityManager, CRUD $crud, NovoCodigo $novoCodigo) {
        $this->crud = $crud;
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->novoCodigo = $novoCodigo;

    }

    private function updateNormalFeeding($normalFeeding) {
        $sth = $this->connection->prepare(self::UPDATE_NORMAL_FEEDING);
        // $observation = array_key_exists("observation", $normalFeeding) ? $normalFeeding['observation'] : null;
        $observation = \Zeedhi\Framework\Util\Functions::arrayKeyExists("observation", $normalFeeding) ? $normalFeeding['observation'] : null;
        $observation = Validation::removeAccentuation($observation);
        $idescoconf = $normalFeeding['selectedByUser'] ? 'S' : 'N';
        $idrestconf = $normalFeeding['selectedByUser'] == false ? '0' : $normalFeeding['idrestconf']; 
        $sth->bindParam(':IDESCOCONF', $idescoconf);
        $sth->bindParam(':IDRESTCONF', $idrestconf);
        $sth->bindParam(':DSOBSCONF', $observation, \PDO::PARAM_STR);
        $sth->bindParam(':OLD_CDFILIAL', $normalFeeding['unit']);
        $sth->bindParam(':OLD_CDPACIENTE', $normalFeeding['id']);
        $sth->bindParam(':OLD_DTSERVINTE', $normalFeeding['serviceDate']);
        $sth->bindParam(':OLD_NRSEQUSERV', $normalFeeding['sequential']);
        $sth->bindParam(':OLD_CDSEQUCOPR', $normalFeeding['unique']);

        $sth->execute();

    }
    
    private function getNormalFeeding($normalFeeding) {
        
        $sth = $this->connection->prepare("
        SELECT COUNT(1) FROM CONFESCO WHERE CDPACIENTE = :PATIENT AND CDFILIAL = :UNIT AND DTSERVINTE = :DTSERVINTE AND CDSEQUCOPR = :CDSEQUCOPR
        ");

        $sth->execute(array(
            'PATIENT' => $normalFeeding['id'],
            'UNIT' => $normalFeeding['unit'],
            'DTSERVINTE' => $normalFeeding['serviceDate'],
            'CDSEQUCOPR' => $normalFeeding['unique']
        ));

        return $sth->fetch(\PDO::FETCH_ASSOC);
    }

    private function getSequential($normalFeeding) {
        $sth = $this->connection->prepare(self::GET_SEQUENTIAL_SERVINTE);
       
        $sth->execute(array(
            'PATIENT' => $normalFeeding['id'],
            'UNIT' => $normalFeeding['unit'],
            'SERVICE_DATE' => $normalFeeding['serviceDate'],
            'DTINCLUSAO' => $normalFeeding['DTINCLUSAO']
        ));
        return current($sth->fetch(\PDO::FETCH_NUM));

    }
    
    private function getSequentialConfesco($normalFeeding) {
        
        $sth = $this->connection->prepare(self::GET_SEQUENTIAL_CONFESCO);
        $serviceDate = explode(' ', $normalFeeding['serviceDate'])[0];
        $sth->execute(array(
            'PATIENT' => $normalFeeding['id'],
            'UNIT' => $normalFeeding['unit'],
            'SERVICE_DATE' => $serviceDate
        ));

        return $sth->fetch(\PDO::FETCH_NUM);
    }

    private function insertNormalFeeding($normalFeeding) {
        $sequential = $this->getSequential($normalFeeding);
        $cdsequcopr = $this->novoCodigo->getCodeByTableName('CONFESCO', 1);
        $sth = $this->connection->prepare(self::INSERT_NORMAL_FEEDING);
        // $observation = array_key_exists("observation", $normalFeeding) ? $normalFeeding['observation'] : null;
        $observation = \Zeedhi\Framework\Util\Functions::arrayKeyExists("observation", $normalFeeding) ? $normalFeeding['observation'] : null;
        $sth->execute(array(
                'CDPACIENTE' => $normalFeeding['id'],
                'DTSERVINTE' => $normalFeeding['serviceDate'],
                'NRSEQUSERV' => $sequential,
                'IDESCOCONF' => $normalFeeding['selectedByUser'] ? 'S' : 'N',
                'IDOPCAOCONF'=> $normalFeeding['isRequiredOnMenu'] ? 'N' : 'S',
                'CDSEQUCOPR' => $cdsequcopr,
                'CDFILIAL' => $normalFeeding['unit'],
                'CDPRATO' => $normalFeeding['formulationId'],
                'CDTIPOPRAT' => $normalFeeding['plateType'],
                'CDFILAUXPRAT' => 'P',
                'DSOBSCONF' => $observation
        ));
        return $cdsequcopr;
    }

    public function save($normalFeedings) {
        $erroToSaveNormalFeeding = array(
            'NORMALFEEDING' => array(),
            'ERROR' => array(),
            'OPERATION' => array(),
            'SEQUENTIAL' => array()
        );
        
        foreach ($normalFeedings as $normalFeeding) {
            if($normalFeeding['__is_new']) {
                try{
                    $sequential = $this->insertNormalFeeding($normalFeeding);
                    array_push($erroToSaveNormalFeeding['SEQUENTIAL'], $sequential);
                }catch(\Exception $e){
                    array_push($erroToSaveNormalFeeding['NORMALFEEDING'], $normalFeeding);
                    array_push($erroToSaveNormalFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveNormalFeeding['OPERATION'], 'INSERT');
                    array_push($erroToSaveNormalFeeding['SEQUENTIAL'], null);
                }
            }else{
                try{
                    $this->updateNormalFeeding($normalFeeding);
                } catch(\Exception $e){
                    array_push($erroToSaveNormalFeeding['NORMALFEEDING'], $normalFeeding);
                    array_push($erroToSaveNormalFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveNormalFeeding['OPERATION'], 'UPDATE');
                }
            }
        }

        return $erroToSaveNormalFeeding;
    }
    
    public function delete($feedingsDelete){
        $tableName = "CONFESCO";
        $erroToDeleteNormalFeeding = array(
            'NORMALFEEDING' => array(),
            'ERROR' => array(),
            'OPERATION' => 'DELETE'
        );
        foreach ($feedingsDelete as $feeding){
            if(isset($feeding["unique"])){
                $feeding["cdfilial"] = $feeding["unit"];
                $feeding["cdsequcopr"] = $feeding["unique"];
                $feeding["nrsequserv"] = $feeding["sequential"];
                $feeding["dtservinte"] = $feeding["serviceDate"];
                $feeding["cdpaciente"] = $feeding["id"];
                try{
                    if(isset($feeding['inconsistenceToDelete']) && $feeding['inconsistenceToDelete'] == true){
                        throw new \Exception('Delete propagado, porém houve erro ao remover registro pai');
                    }
                    $this->crud->delete($tableName, $feeding);
                } catch (\Exception $e){
                    array_push($erroToDeleteNormalFeeding['NORMALFEEDING'], $feeding);
                    array_push($erroToDeleteNormalFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                }
            }
        }
        return $erroToDeleteNormalFeeding;
    }
}