<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use TecDiet\Util\Validation;
use TecDiet\Services\VerificaRestricaoAlimentar;

class PatientsService implements Queries {

    protected $entityManager;
    protected $connection;
    protected $patients = array();
    
    protected $auxPatientSequential;
    protected $defaultPatientCode;
    protected $verificaRestricaoAlimentar;

    public function __construct(EntityManager $entityManager, VerificaRestricaoAlimentar $verificaRestricaoAlimentar) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->verificaRestricaoAlimentar = $verificaRestricaoAlimentar;

    }

    private function updateHospitalization($patient) {
        $updateInternac = $this->connection->prepare(self::UPDATE_INTERNAC);
        if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('oldSituation', $patient)){
            $situation = $patient['oldSituation'];
        }else{
            $situation = $patient['situation'];
        }
        
        $updateInternac->execute(
            array(
                'SITUATION' => $situation,
                'DISCHARGEDATE' => $patient['dischargeDate'],
                'PATIENT' => $patient['id'],
                'UNITS' => $patient['unit']
            )
        );
    }
    
    private function updatePatientBed($patient) {
        $updateInternac = $this->connection->prepare(self::UPDATE_INTERNACLEITO);
        
        $updateInternac->execute(
            array(
                'CDLEITO' => $patient['bed'],
                'PATIENT' => $patient['id'],
                'UNITS' => $patient['unit']
            )
        );
    }
    private function updateBedStatus($patient, $bedStatus) {
        $sth = $this->connection->prepare(self::UPDATE_OCUPALEITO);
        $sth->execute(
            array(
                'IDOCUPLEITO' => $bedStatus,
                'PATIENT' => $patient['id'],
                'UNITS' => $patient['unit']
            )
        );
    }
    
    public function updateFoodRestriction($rows, $response) {
        foreach($rows as $row){
            $this->verificaRestricaoAlimentar->executeVerificaRestricaoAlimentar($row, $response);
        }
    }

    public function save($patients) {
        $erroToSavePatient = array(
            'PATIENT' => array(),
            'ERROR' => array(),
            'OPERATION' => array()
        );
        foreach ($patients as $patient) {
            if($patient["__is_new"]){
                $errorToInsertNewPatient = false;
                try{
                    self::insertPatient($patient);
                } catch(\Exception $e){
                    $errorToInsertNewPatient = true;
                    $patient['GENERIC_MESSAGE'] = nl2br('Erro ao inserir paciente<br />'.$patient['formattedName']);
                    array_push($erroToSavePatient['PATIENT'], $patient);
                    array_push($erroToSavePatient['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSavePatient['OPERATION'], 'INSERT');
                }
                if(!$errorToInsertNewPatient && \Zeedhi\Framework\Util\Functions::arrayKeyExists('newHospitalization', $patient) && $patient['newHospitalization']){
                    $this->connection->beginTransaction();
                    try{
                        $this->insertHospitalization($patient);
                        $this->updateBedStatus($patient, 'N');
                        $this->updatePatientBed($patient);
                        $this->updateBedStatus($patient, 'S');
                        $this->connection->commit();
                    } catch (\Exception $e){
                        $this->connection->rollback();
                        $patient['GENERIC_MESSAGE'] = nl2br('Erro ao internar paciente<br />'.$patient['formattedName']);
                        array_push($erroToSavePatient['PATIENT'], $patient);
                        array_push($erroToSavePatient['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                        array_push($erroToSavePatient['OPERATION'], 'UPDATE');
                    }
                }
            } else {
                    if($patient['situation'] !== 'INTERNADO') {
                        $this->connection->beginTransaction();
                        if($patient['__is_new'] == false && $patient['newHospitalization'] ==true){
                            try {
                                $this->insertHospitalization($patient, 0);
                                $this->connection->commit();
                            } catch (\Exception $e) {
                                $situation = 'alta';
                                $patient['GENERIC_MESSAGE'] = nl2br('Erro ao declarar '.$situation.' para o paciente<br / >'.$patient['formattedName']);
                                array_push($erroToSavePatient['PATIENT'], $patient);
                                array_push($erroToSavePatient['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                                array_push($erroToSavePatient['OPERATION'], 'UPDATE');
                            }
                        } else {
                            try {
                                $this->updateHospitalization($patient);
                                $this->updateBedStatus($patient, 'N');
                                $this->connection->commit();
                            } catch (\Exception $e) {
                                $this->connection->rollback();
                                $situation = '';
                                if($patient['situation'] == 0){
                                    $situation = 'alta';
                                } else if($patient['situation'] == 1){
                                    $situation = 'óbito';
                                }
                                $patient['GENERIC_MESSAGE'] = nl2br('Erro ao declarar '.$situation.' para o paciente<br / >'.$patient['formattedName']);
                                array_push($erroToSavePatient['PATIENT'], $patient);
                                array_push($erroToSavePatient['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                                array_push($erroToSavePatient['OPERATION'], 'UPDATE');
                            }
                        }
                    } else {
                        $this->connection->beginTransaction();
                        
                        try{
                            if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('newHospitalization', $patient) && $patient['newHospitalization']){
                                if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('oldSituation', $patient)){
                                    $this->updateHospitalization($patient);
                                }
                                $this->insertHospitalization($patient, null);
                            }
                            if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('oldBed', $patient)){
                                $this->updateLeito($patient);
                            }
                            $this->updatePatientBed($patient);
                            $this->updateBedStatus($patient, 'S');
                            $this->connection->commit();
                        } catch (\Exception $e){
                            $this->connection->rollback();
                            if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('newHospitalization', $patient) && $patient['newHospitalization']){
                                $patient['GENERIC_MESSAGE'] = nl2br('Erro ao internar paciente<br />'.$patient['formattedName']);
                            }else{
                                $patient['GENERIC_MESSAGE'] = nl2br('Erro ao alterar leito do paciente <br />'.$patient['formattedName']);
                            }
                            array_push($erroToSavePatient['PATIENT'], $patient);
                            array_push($erroToSavePatient['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                            array_push($erroToSavePatient['OPERATION'], 'UPDATE');
                        }
                    }
            }
        }
        return $erroToSavePatient;
    }
    public function updateLeito($patient){
        $sth = $this->connection->prepare(self::UPDATE_LEITO);
        $bed = $patient['oldBed']['bed'];
        $sth->execute(
            array(
                'CDLEITO' => $bed,
                'CDFILIAL' => $patient['unit']
            )
        );
    }
    
    public function insertHospitalization($patient, $idmotialta){
        $sth = $this->connection->prepare(self::INSERT_INTERNAC);
        $id = $idmotialta === 0 ? $idmotialta : null;
        $date = $idmotialta === 0 ? $patient['dischargeDate'] : null;

        $insertParams = array(
            "CDFILIAL"      => $patient['unit'],
            "CDPACIENTE"    => $patient['id'],
            "DTINTERNA"     => $patient['hospitalization'],
            "DTALTA"        => $date,
            "CDEQUIMED"     => $patient['medicalTeamId'],
            "CDSERPADDIE"   => $patient['healthInsuranceId'],
            "DSPRESMEDI"    => null,
            "CDLEITO"       => $patient['bed'],
            "IDMOTIALTA"    => $id,
            "NRORG"         => $patient['organizationNumber'],
            "DTINCLUSAO"    => $patient['DTINCLUSAO'],
            "DTULTATU"      => $patient['DTULTATU'],
            "NRORGINCLUSAO" => $patient['organizationNumber'],
            "CDOPERINCLUSAO"=> $patient['CDOPERINCLUSAO'],
            "NRORGULTATU"   => $patient['organizationNumber'],
            "CDOPERULTATU"  => $patient['CDOPERULTATU'],
            "IDATIVO"       => 'S'
        );
        $sth->execute($insertParams);
    }
    
    public function insertPatient($patient) {
        
        $this->auxPatientSequential = 0;
        $this->defaultPatientCode = $patient['id'];
        
        $sth = $this->connection->prepare(self::INSERT_PATIENT);
        
        $insertParams = array(
            "CDFILIAL"      => $patient['unit'],
            "CDPACIENTE"    => $patient['id'],
            "NMPACIENTE"    => ucwords(strtolower($patient['name'])),
            "NMSOBRNOME"    => ucwords(strtolower($patient['lastName'])),
            "NMPACIUPPER"   => ucwords(strtolower($patient['name'])). ' ' .strtoupper($patient['lastName']),
            "DTNASCPACI"    => $patient['birthDate'],
            "IDSEXOPACI"    => $patient['gender'] == 'M' ? 0 : 1,
            "NRORG"         => $patient['organizationNumber']
        );
        
        $sth->execute($insertParams);

    }
    
    public function patientSequentialAjusta($unitsId){
        
        $sth = $this->connection->prepare(self::IS_SEQUENTIAL_AJUSTA);
        $params = array(
            'CDFILIAL' => $unitsId
        );
	    $sth->execute($params);  
	    
	    $isSequentialPatientAjusta = $sth->fetch(\PDO::FETCH_ASSOC);
	    
	    if(!$isSequentialPatientAjusta){
	        return [];
	    }
	    return $isSequentialPatientAjusta;
    } 
    
    public function getPatientSequential($row){
        $sth = $this->connection->prepare(self::GET_COUNT_PATIENT_CODE);
        $selectParams = array(
            "CDFILIAL"      => $row["unit"]
        );
        $sth->execute($selectParams);
        $countPatientEqualCodes = $sth->fetchAll()[0]["COUNT_PATIENT_CODE"];
        if($row["isAjusta"] === 'S'){
            $countPatientEqualCodes = str_pad($countPatientEqualCodes, 12, "0", STR_PAD_LEFT);
        }
        $codePatient = array(
            "CDPACIENTE" => $countPatientEqualCodes
        );
        return $codePatient;
    }

    private function getNormalFeedings($patient) {
        $sth = $this->connection->prepare(self::NORMAL_FEEDING);

        $sth->execute(array(
            'PATIENT' => $patient['CDPACIENTE'],
            'UNIT' => $patient['CDFILIAL']
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }
    
    private function getRestrictionAndIntolerance($feeding) {
        $sth = $this->connection->prepare(self::GET_RESTINT_BY_PLATE);
        
        $sth->execute(array(
            'CDFILAUXPRAT' => $feeding['CDFILAUXPRAT'],
            'CDPRATO' => $feeding['CDPRATO'],
            'CDFILIAL' => $feeding['CDFILIAL'],
            'CDSERVFILI' => $feeding['CDSERVFILI'],
            'CDPACIENTE' => $feeding['CDPACIENTE'],
            'DTINTERNA' => $feeding['DTINTERNA']
        ));
        return $sth->fetchAll(\PDO::FETCH_ASSOC);
    }
    
    private function getPlatesIntoRestriction($unit){
        $sth = $this->connection->prepare(self::GET_PLATES_RESINT);
        
        $sth->execute(array(
            'CDFILIAL' => $unit
        ));
        return $sth->fetchAll(\PDO::FETCH_ASSOC);
    }

    private function getLactaryFeedings($patient) {

        $sth = $this->connection->prepare(self::LACTARY_FEEDING);

        $sth->execute(array(
            'PATIENT' => $patient['CDPACIENTE'],
            'UNIT' => $patient['CDFILIAL']
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getEnteralFeedings($patient) {

        $sth = $this->connection->prepare(self::ENTERAL_FEEDING);

        $sth->execute(array(
            'PATIENT' => $patient['CDPACIENTE'],
            'UNIT' => $patient['CDFILIAL']
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getControlFeedings($patient) {

        $sth = $this->connection->prepare(self::CONTROL_FEEDING);

        $sth->execute(array(
            'PATIENT' => $patient['CDPACIENTE'],
            'UNIT' => $patient['CDFILIAL']
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getBioEvaluation($patient, $appointmentDate) {

        $sth = $this->connection->prepare(self::BIOCHEMICAL_EVALUATION);
        
        $sth->execute(array(
            'EXAMINATION' => 'T',
            'APPOINTMENT_DATE' => $appointmentDate,
            'UNIT' => $patient['CDFILIAL'],
            'PATIENT' => $patient['CDPACIENTE']
        ));
        
        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getDataInfoEvaluations($patient, $appointmentDate) {

        $sth = $this->connection->prepare(self::DATA_INFO_EVALUATION);

        $sth->execute(array(
            'UNIT' => $patient['CDFILIAL'],
            'PATIENT' => $patient['CDPACIENTE'],
            'APPOINTMENT_DATE' => $appointmentDate
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getAnthMethods($patient, $appointmentDate) {

        $dateTime = explode(' ', $appointmentDate);
        $dateTime = implode("-", array_reverse(explode("/", $dateTime[0]))).' '.$dateTime[1];
        $age = floor(abs(strtotime($dateTime) - strtotime($patient['DTNASCPACI']))/60/60/24/365);

        $sth = $this->connection->prepare(self::ANTHROPOMETRIC_EVALUATION);
        $sth->execute(array(
            'AGE' => $age,
            'APPOINTMENTDATE' => $appointmentDate,
            'UNITS' => $patient['CDFILIAL'],
            'PATIENT' => $patient['CDPACIENTE']
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getAnthGeneral($patient) {
        $sth = $this->connection->prepare(self::ANTHROPOMETRIC_EVALUATION_GENERAL);
        $sth->execute(array(
            'P_CDFILIAL' => $patient['CDFILIAL'],
            'P_CDPACIENTE' => null,
            'P_DTCONSPACI' => null
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getAppointments($patient) {

        $sth = $this->connection->prepare(self::APPOINTMENTS);

        $sth->execute(array(
            'PATIENT' => $patient['CDPACIENTE'],
            'UNITS' => $patient['CDFILIAL']
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getPrescriptions($patient) {

        $sth = $this->connection->prepare(self::PRESCRIPTIONS);

        $sth->execute(array(
            'PATIENT' => $patient['CDPACIENTE'],
            'UNITS' => $patient['CDFILIAL']
        ));
        
        return $sth->fetchAll(\PDO::FETCH_ASSOC);
    }

    public function getProfile($patient) {

        $sth = $this->connection->prepare(self::PROFILES);

        $sth->execute(array(
            'PATIENT' => $patient['CDPACIENTE'],
            'UNITS' => $patient['CDFILIAL']
        ));

        return $sth->fetch(\PDO::FETCH_ASSOC);

    }
    public function getResult($patient, $dataConsulta) {

        $sth = $this->connection->prepare(self::RESULT_EVALUATION);
        $sth->execute(array(
            'CDPACIENTE' => $patient['CDPACIENTE'],
            'CDFILIAL' => $patient['CDFILIAL'],
            'DTCONSPACI' => $dataConsulta
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }
    
    private function initPatient($unitsID, $nutritionistID) {
        $sth = $this->connection->prepare(self::HAS_NUTRITIONIST_PARAMS);

        $sth->execute(array(
            'CDFILIAL'  => $unitsID,
            'CDNUTRICIO'=> $nutritionistID
        ));
        $has_params = $sth->fetchAll(\PDO::FETCH_ASSOC);
        
        if($has_params){
            $sth = $this->connection->prepare(self::NUTRITIONIST_PATIENTS);
    
            $sth->execute(array(
                'CDFILIAL' => $unitsID,
                'CDNUTRICIO'=> $nutritionistID
            ));
        }else{
            $sth = $this->connection->prepare(self::PATIENTS);

            $sth->execute(array(
                'CDFILIAL' => $unitsID
            ));
        }
        
        $patients = $sth->fetchAll(\PDO::FETCH_ASSOC);
        
        $keyDataInfoEvaluation = 0;
        $keyMethods = 0;
        $keyPrescriptions = 0;
        $keyMethodsGeneral = 0;
        $keyResults = 0;
        $keyBioEvaluation = 0;
        
        $platesIntoRestriction = $this->getPlatesIntoRestriction($unitsID);
        foreach($platesIntoRestriction as $plate){
            $this->patients['restrictionAndIntolerance'][] = array(
                'id' => $plate['CDPACIENTE'],
                'CDPACIENTE' => $plate['CDPACIENTE'], 
                'CDFILIAL' => $plate['CDFILIAL'], 
                'CDPRATO' => $plate['CDPRATO'], 
                'CDTIPOPRAT' => $plate['CDTIPOPRAT'], 
                'CDFILAUXPRAT' => $plate['CDFILAUXPRAT'], 
                'NRSEQUSERV' => $plate['NRSEQUSERV'], 
                'CDSERVFILI' => $plate['CDSERVFILI'], 
                'DTINTERNA' => $plate['DTINTERNA'],
                'DTSERVINTE' => $plate['DTSERVINTE'],
                'INTORESTRICTION' => $this->getRestrictionAndIntolerance($plate)
            );
        }
            
        foreach ($patients as $patient) {

            $age = Validation::getAge($patient['DTNASCPACI']);
            $ageNumber = filter_var($age, FILTER_SANITIZE_NUMBER_INT);

            $this->patients['init'][] = array(
                'id' => $patient['CDPACIENTE'],
                'age' => Validation::getYears($patient['DTNASCPACI']),
                'yearAndMonth' => Validation::getAge($patient['DTNASCPACI']),
                'birthDate' => $patient['DTNASCPACI'],
                'formattedName' => $patient['NMPACIUPPER'],
                'normalizedName' => $patient['CDPACIENTE'].Validation::removeAccents($patient['NMPACIUPPER']),
                'idnamepaci' => $patient['IDNAMEPACI'],
                'situation' => $patient['IDMOTIALTA'],
                'storey' => $patient['CDANDAR'],
                'hall' => $patient['CDALA'],
                'bed' => $patient['CDLEITO'],
                'bedName' => $patient['NMLEITO'],
                'dischargeDate' => $patient['DTALTA'],
                'hospitalization' => $patient['DTINTERNA'],
                'unit' => $patient['CDFILIAL'],
                'gender' => $patient['IDSEXOPACI'],
                'lastVisit' => $patient['DTULTATU'],
                'newHospitalization' => false
            );

            // PATIENT PROFILE
            $profile = $this->getProfile($patient);
            
            $this->patients['profiles'][] = array(
                'id' => $patient['CDPACIENTE'],
                'unit' => $patient['CDFILIAL'],
                'confidentiality' => ($profile !== false) ? $profile['DSOBSCONFID'] : '',
                'observation' => ($profile !== false) ? $profile['DSOBSGERAL'] : '',
                'eatingHabits' => Validation::toPlainText(utf8_encode(($profile !== false) ? $profile['TXHABIALIM']: '')),
                'foodIntolerance' => Validation::toPlainText(utf8_encode(($profile !== false) ? $profile['TXINTOALIM']: ''))
            );

            // PATIENT PRESCRIPTIONS
            $prescriptions = $this->getPrescriptions($patient);
            foreach ($prescriptions as $prescription) {
                $this->patients['prescriptions'][] = array(
                    'id' => $patient['CDPACIENTE'],
                    'unit' => $patient['CDFILIAL'],
                    'bed' => $prescription['CDLEITO'],
                    'bedName' => $prescription['NMLEITO'],
                    'storey'    => $prescription['CDANDAR'],
                    'storeyName'    => $prescription['NMANDAR'],
                    'hall'  => $prescription['CDALA'],
                    'hallName'  => $prescription['NMALA'],
                    'oldBed' => $patient['CDLEITO'],
                    'sequential' => $keyPrescriptions,
                    'date' => $prescription['DTPRESDIET'],
                    'dietId' => $prescription['CDDIETPAD'],
                    'oldDietId' => $prescription['CDDIETPAD'],
                    'dietName' => $prescription['DSDIETPAD'],
                    'nutritionalStatusId' => $prescription['CDNIASNUTR'],
                    'nutritionalStatusName' => $prescription['NMNIASNUTR'],
                    'prescription' => $prescription['DSPRESDIET'],
                    'medicalPrescription' => $prescription['DSPRESMEDDIE'],
                    'clinical' => $prescription['DSDADOCLIN']
                );
                $keyPrescriptions++;
            } 
            
            // PATIENT APPOINTMENT
            $appointments = $this->getAppointments($patient);
            foreach ($appointments as $appointment) {
                
                $appointment['ALTURA'] = Validation::sanitazeNumber($appointment['ALTURA'], true);
                $appointment['PESO'] = Validation::sanitazeNumber($appointment['PESO'], true);
                $this->patients['appointments'][] = array(
                    'id' => $patient['CDPACIENTE'],
                    'unit' => $patient['CDFILIAL'],
                    'sequential' => $appointment['ROWNUM'],
                    'date' => $appointment['DTCONSPACI'],
                    'height' => $appointment['ALTURA'],
                    'weight' => $appointment['PESO'],
                    'dietId' => $appointment['CDDIETPAD'],
                    'dietName' => $appointment['DSDIETPAD'],
                    'healthInsuranceId' => $appointment['CDSERPADDIE'],
                    'healthInsuranceName' => $appointment['NMSERPADDIE'],
                    'nutritionalStatusId' => $appointment['CDNIASNUTR'],
                    'nutritionalStatusName' => $appointment['NMNIASNUTR'],
                    'medicalTeamId' => $appointment['CDEQUIMED'],
                    'medicalTeamName' => $appointment['NMEQUIMED'],
                    'nutritionistId' => $appointment['CDOPERADOR'],
                    'nutritionistName' => $appointment['NMOPERADOR']
                );
                
                $anthResults = $this->getResult($patient, $appointment['DTCONSPACI']);
                
                foreach ($anthResults as $result) {
                    $this->patients['anthAvaliationResults'][] = array(
                        'id'             => $result['CDPACIENTE'],
                        'idMetodo'       => $result['CDTBMETOAVAL'],
                        'appointmentDate'=> $result['DTCONSPACI'],
                        'anthDate'       => $result['DTINCLUSAO'],
                        'nomeResul'      => $result['DSCONTRESU'],
                        'resultado'      => $result['DSRESUAVAL'],
                        'nomeMetodo'     => $result['NMTBMETOAVAL'],
                        'key'            => $keyResults
                    );
                    $keyResults++;
                }

                // ANTHROPOMETRIC EVALUATION
                $anthMethods = $this->getAnthMethods($patient, $appointment['DTCONSPACI']);
                foreach($anthMethods as $anthMethod) {
                    $this->patients['anthEvaluationMethods'][] = array(
                        'id' => $anthMethod['CDPACIENTE'],
                        'unit' => $anthMethod['CDFILIAL'],
                        'key' => (String)$keyMethods,
                        'appointmentDate' => $appointment['DTCONSPACI'],
                        'reference' => $anthMethod['CDTABELACO'],
                        'name' => $anthMethod['NMTBMETOAVAL'],
                        'method' => $anthMethod['CDTBMETOAVAL'],
                        'date' => $anthMethod['DTCONSPACI'],
                        '__isSelected' => Validation::isSelected($anthMethod['IDTABELAAV'])
                    );

                    $keyMethods++;
                }
                $anthGenerals = $this->getAnthGeneral($patient);
                foreach($anthGenerals as $anthGeneral) {
                    $this->patients['anthEvaluationGeneral'][] = array(
                        'key' => (String)$keyMethodsGeneral,
                        'unit' => $anthGeneral['CDFILIAL'],
                        'id' => $anthGeneral['CDPACIENTE'],
                        'appointmentDate' => $anthGeneral['DTCONSPACI'],
                        '__isSelected' => Validation::isSelected($anthGeneral['IDTABELAAV']),
                        'reference' => $anthGeneral['CDTABELACO'],
                        'methodEvaluationId' => $anthGeneral['CDTBMETOAVAL'],
                        'name' => $anthGeneral['NMTBMETOAVAL'],
                        'dataIni' => intval($anthGeneral['VRIDADEINITB']),
                        'dataFim' => intval($anthGeneral['VRIDADEFINTB'])
                    );
                    $keyMethodsGeneral++;
                }
                
                
                // BIOCHEMICAL EVALUATION
                $bioEvaluations = $this->getBioEvaluation($patient, $appointment['DTCONSPACI']);
                
                foreach($bioEvaluations as $bioEvaluation) {
                    $bioEvaluation['VRRESEXA'] = Validation::sanitazeNumber($bioEvaluation['VRRESEXA'], true); 
                    
                    $this->patients['bioEvaluations'][] = array(
                        'id' => $patient['CDPACIENTE'],
                        'unit' => $patient['CDFILIAL'],
                        'gender' => $patient['IDSEXOPACI'],
                        'age' => $ageNumber,
                        'key' => (string)$keyBioEvaluation,
                        'priority' => $bioEvaluation['NRSEQEXAMREA'],
                        'examinationId' => $bioEvaluation['CDEXAME'],
                        'oldExaminationId' => $bioEvaluation['CDEXAME'],
                        'examinationName' => $bioEvaluation['NMEXAME'],
                        'examinationDate' => $bioEvaluation['DTREALEX'],
                        'examinationUnit' => $bioEvaluation['CDUNIDEXE'],
                        'examinationValue' => $bioEvaluation['VRRESEXA'],
                        'examinationStatus' => $bioEvaluation['DSRECLASSI'],
                        'examinationFormula' => $bioEvaluation['TXFORMEXAM'],
                        'appointmentDate' => $bioEvaluation['DTCONSPACI'],
                        'pregnancyWeek' => $bioEvaluation['NRSEMGEST'],
                        'pregnancyId' => $bioEvaluation['IDGRAVIDEZ'],
                        'pregnancyName' => Validation::getPregnancyName($bioEvaluation['IDGRAVIDEZ'])
                    );

                    $keyBioEvaluation++;

                }

                $dataInfoEvaluations = $this->getDataInfoEvaluations($patient, $appointment['DTCONSPACI']);

                foreach($dataInfoEvaluations as $dataInfoEvaluation) {
                    $dataInfoEvaluation['DSINFOAVAL'] = Validation::sanitazeNumber($dataInfoEvaluation['DSINFOAVAL'], true);

                    $this->patients['dataInfoEvaluations'][] = array(
                        'key' => (String)$keyDataInfoEvaluation,
                        'id' => $patient['CDPACIENTE'],
                        'unit' => $patient['CDFILIAL'],
                        'value' => $dataInfoEvaluation['DSINFOAVAL'],
                        'methodEvaluationId' => $dataInfoEvaluation['CDTBMETOAVAL'],
                        'dataTypeEvaluationId' => $dataInfoEvaluation['CDTIPODADO'],
                        'appointmentDate' => $dataInfoEvaluation['DTCONSPACI']
                    );

                    $keyDataInfoEvaluation++;
 
                }

            }

            // CONTROL FEEDING

            $controlFeedings = $this->getControlFeedings($patient);

            foreach ($controlFeedings as $controlFeeding) {

                $this->patients['controlFeedings'][] = array(
                    // 'id' => $patient['CDPACIENTE'],
                    // 'unit' => $patient['CDFILIAL'],
                    // 'serviceId' => $controlFeeding['IDFORMDIET'],
                    // 'service' => $controlFeeding['NMSERVICO'],
                    // 'serviceName' => Validation::getControlFeedingServiceName($controlFeeding['IDFORMDIET']),
                    // 'dietName' => $controlFeeding['DSDIETPAD'],
                    // 'serviceDate' => $controlFeeding['DTSERVINTE'],
                    // 'sequential' => $controlFeeding['NRSEQUSERV'],
                    // 'stpaciacom' => Validation::getControlFeedingRequester($controlFeeding['STPACIACOM'])

                    'id' => $patient['CDPACIENTE'],
                    'unit' => $patient['CDFILIAL'],
                    'cdservfili' => $controlFeeding['CDSERVFILI'],
                    'dietTypeName' => Validation::getControlFeedingServiceName($controlFeeding['IDFORMDIET']),
                    'serviceName' => $controlFeeding['NMSERVICO'],
                    'dietId' => $controlFeeding['CDDIETPAD'],
                    'dietName' => $controlFeeding['DSDIETPAD'],
                    'serviceDate' => $controlFeeding['DTSERVINTE'],
                    'DTINCLUSAO' => $controlFeeding['DTINCLUSAO'],
                    'sequential' => $controlFeeding['NRSEQUSERV'],
                    'idrestserin' => $controlFeeding['IDRESTSERIN'],
                    'idformdiet' => $controlFeeding['IDFORMDIET'],
                    'idtpservhosp' => $controlFeeding['IDTPSERVHOSP'],
                    'solicitante' => Validation::getControlFeedingRequester($controlFeeding['STPACIACOM'])
                    
                );

            }

            $normalFeedings = $this->getNormalFeedings($patient);
    
            foreach ($normalFeedings as $normalFeeding) {

                $this->patients['normalFeedings'][] = array(
                    'id' => $patient['CDPACIENTE'],
                    'unit' => $patient['CDFILIAL'],
                    'serviceId' => '0',
                    'key' => $normalFeeding['ROWNUM'].$patient['CDPACIENTE'],
                    'unique' => $normalFeeding['CDSEQUCOPR'],
                    'plateName' => $normalFeeding['NMPRATO'],
                    'plateType' => $normalFeeding['CDTIPOPRAT'],
                    'plateTypeName' => $normalFeeding['NMTIPOPRAT'],
                    'serviceDate' => $normalFeeding['DTSERVINTE'],
                    'plateNumbers' => $normalFeeding['QTPRATCONF'],
                    'plateAuxId' => $normalFeeding['CDFILAUXPRAT'],
                    'formulationId' => $normalFeeding['CDPRATO'],
                    'observation' => $normalFeeding['DSOBSCONF'],
                    'sequential' => $normalFeeding['NRSEQUSERV'],
                    'dietId' => $normalFeeding['CDDIETPAD'],
                    'isRequiredOnMenu' => $normalFeeding['IDOPCAOCONF'] === 'N' ? true : false,
                    'selectedByUser' => $normalFeeding['IDESCOCONF'] == 'S' ? true : false,
                    'idrestconf' => $normalFeeding['IDRESTCONF'],
                    'idopccnfcard' => $normalFeeding['IDOPCCNFCARD'],
                );
                
    
            }
            
            $enteralFeedings = $this->getEnteralFeedings($patient);

            foreach ($enteralFeedings as $enteralFeeding) {

                if($enteralFeeding['IDPREPCONF'] == 'A') {
                    $formulationName = $enteralFeeding['NMPRATO'];
                    $formulationId = $enteralFeeding['CDPRATO'];
                }else {
                    $formulationName = $enteralFeeding['NMPRODUTO'];
                    $formulationId = $enteralFeeding['CDPRODUTO'];
                }
                // $qtpratconf = $enteralFeeding['QTPRATCONF'][0] === '.' || $enteralFeeding['QTPRATCONF'][0] === ',' ? '0'.$enteralFeeding['QTPRATCONF'] : $enteralFeeding['QTPRATCONF'];
                // $qtinfuconf = $enteralFeeding['QTINFUCONF'][0] === '.' || $enteralFeeding['QTINFUCONF'][0] === ',' ? '0'.$enteralFeeding['QTINFUCONF'] : $enteralFeeding['QTINFUCONF'];
                // $qtservconf = $enteralFeeding['QTSERVCONF'][0] === '.' || $enteralFeeding['QTSERVCONF'][0] === ',' ? '0'.$enteralFeeding['QTSERVCONF'] : $enteralFeeding['QTSERVCONF'];
                $this->patients['enteralFeedings'][] = array(
                    'id' => $patient['CDPACIENTE'],
                    'unit' => $patient['CDFILIAL'],
                    'serviceId' => '2',
                    'key' => $enteralFeeding['ROWNUM'].$patient['CDPACIENTE'],
                    'unique' => $enteralFeeding['CDSEQUCOPR'],
                    'formulationName' => $formulationName,
                    'accessRoadId' => $enteralFeeding['CDVIACESDIET'],
                    'accessRoadName' => $enteralFeeding['NMVIACESDIET'],
                    'serviceDate' => $enteralFeeding['DTSERVINTE'],
                    'preparationId' => $enteralFeeding['IDPREPCONF'],
                    'preparationName' => Validation::getPreparationName($enteralFeeding['IDPREPCONF']),
                    'unidadeVolume' => $enteralFeeding['UNIDPRA'],
                    'enteralDate' => $enteralFeeding['DTCONSCONF'],
                    'time' => Validation::floatToTime($enteralFeeding['HRCONSCONF']),
                    'volPrescription' => $enteralFeeding['QTPRATCONF'],
                    'plateAuxId' => $enteralFeeding['CDFILAUXPRAT'],
                    'volInfudido' => $enteralFeeding['QTINFUCONF'],
                    'formulationId' => $formulationId,
                    'serviceNumbers' => $enteralFeeding['QTSERVCONF'],
                    'observation' => $enteralFeeding['DSOBSCONF'],
                    'sequential' => $enteralFeeding['NRSEQUSERV']
                );

            }

            $lactaryFeedings = $this->getLactaryFeedings($patient);

            foreach ($lactaryFeedings as $lactaryFeeding) {
                // $qtpratconf = $lactaryFeeding['QTPRATCONF'][0] === '.' || $lactaryFeeding['QTPRATCONF'][0] === ',' ? '0'.$lactaryFeeding['QTPRATCONF'] : $lactaryFeeding['QTPRATCONF'];
                $qtpratconf = $lactaryFeeding['QTPRATCONF'];
                $this->patients['lactaryFeedings'][] = array(
                    'id' => $patient['CDPACIENTE'],
                    'unit' => $patient['CDFILIAL'],
                    'serviceId' => '1',
                    'key' => $lactaryFeeding['ROWNUM'].$patient['CDPACIENTE'],
                    'unique' => $lactaryFeeding['CDSEQUCOPR'],
                    'formulationName' => $lactaryFeeding['NMPRATO'],
                    'serviceDate' => $lactaryFeeding['DTSERVINTE'],
                    'lactaryDate' => $lactaryFeeding['DTCONSCONF'],
                    'hourLactaryPlate' => Validation::floatToTime($lactaryFeeding['HRCONSCONF']),
                    'plateNumbers' => $qtpratconf,
                    'plateAuxId' => $lactaryFeeding['CDFILAUXPRAT'],
                    'formulationId' => $lactaryFeeding['CDPRATO'],
                    'observation' => $lactaryFeeding['DSOBSCONF'],
                    'sequential' => $lactaryFeeding['NRSEQUSERV']
                );

            }

        }

    }

    public function get($unitsID, $nutritionistID) {

        $this->initPatient($unitsID, $nutritionistID);
        return $this->patients;

    }

}