/**
* Creates an instance of RequestEngine
* @class
* @name RequestEngine
* @param {RestEngine} restEngine - An instance of RestEngine
*/
function RequestEngine(restEngine) {
	/**
	* Sends a request via post
	* @function doRequest
	* @memberof RequestEngine
	* @instance
	* @param {Object} request - Object with the request data
	* @param {String} request.requestType - The type of the request to be sent
	* @param {String} request.serviceName - The name of the service that will receive the request
	* @param {Function} [request.processResponse] - A function to be executed as a callback on a success response
	* @param {Function} [request.processError] - A function to be executed as a callback on a error response
	* @param {Number} [request.timeout] - Time in milliseconds to the timeout of the request
	* @param {Number} [request.retryCount] - Number of times to retry sending the request
	* @param {Function} [request.retryCallback] - Callback to be executed when retrying to send the request
	* @returns {Promise} Returns a promise with the data of the request's response. On success and on reject, the promise will also provide the response's data
	*/
	this.doRequest = function(request) {
		var requestParams = {};
		requestParams.service = request.getServiceName();
		requestParams.params = request.getRequestParams();
		requestParams.callBack = request.processResponse;
		requestParams.typeRequest = "post";
		requestParams.errorCallBack = request.processError;
		requestParams.timeout = request.timeout;
		requestParams.returnCount = request.retryCount;
		requestParams.retryCallback = request.retryCallback;
		return restEngine.post(requestParams);
	};
}