function MCListController ($scope, $timeout, ScreenService, eventAggregator, templateManager, localStorageEngine, restEngine, ZHPromise, ApplicationContext) {
    var container = $("#mc-list-container");
    var widget = $scope.widget;
    var keys = ['/Profiles', '/Units', '/Patients', '/Halls', '/Prescriptions', '/NormalFeedings', '/BioEvaluations', '/BedsPrescriptions', '/NutritionalStatus', '/Diets', '/DietsServices', '/Storeys', '/Beds', '/Appointments', '/HealthInsurance', '/MedicalTeams', '/Examinations', '/AnthEvaluationMethods', '/AnthEvaluationGeneral', '/DataTypeEvaluations', '/DataEvaluations', '/ControlFeedings', '/Services', '/LactaryPlates', '/Plates', '/LactaryFeedings', '/EnteralFeedings', '/HallsDistincts', '/PlatesByUnit', '/PlatesPla', '/FoodIntoleranceGroup', '/AllGroupSubgroup', '/PatientIntoRestriction', '/EnteralPlates', '/EnteralProducts', '/OralServices', '/LactaryServices', '/EnteralServices'];

    function addData(patient) {
        var datasources = widget.container.getWidget('WidgetDetailsPatient').localDataSourceLoaded;
        var searchForId = patient.id;
        var dataToBeAnexed = [];
        keys.forEach(function (keyDirty) {
            var keyName = keyDirty.replace('/', '');

            if (keyName !== 'Patients' &&
                keyName !== 'NutritionalStatus' &&
                keyName !== 'AnthEvaluationGeneral' &&
                keyName !== 'NormalFeedingsNow' &&
                datasources[keyName] !== null) {

                datasources[keyName].forEach(function (feedInfo) {
                    if (feedInfo.id == searchForId) {
                        dataToBeAnexed.push(feedInfo);
                    }
                });

                patient[keyName] = dataToBeAnexed;
                dataToBeAnexed = [];
            }
        });
        addLocationDataToPrescription(patient);
    }
    
    function addLocationDataToPrescription(patient) {
        if (patient.Prescriptions) {
            patient.Prescriptions.forEach(function (pres, key) {
                var curBedID = pres.bed;
                widget.container.getWidget('WidgetDetailsPatient').localDataSourceLoaded.Beds.forEach(function (beditem) {
                    if (beditem.bed === curBedID) {
                        patient.Prescriptions[key].hall = beditem.hall;
                        patient.Prescriptions[key].storey = beditem.storey;
                    }
                });
            });
        }
    }
    
    this.infiniteScroll = function() {
        var scrollHeight = container[0].scrollHeight - 15;
        var scrollPosition = container.height() + container.scrollTop() - 15; //padding
        var previousScroll = previousScroll || scrollPosition;
        if (((scrollHeight - scrollPosition) / scrollHeight < 0.030) && previousScroll <= scrollPosition) {
            this.loadData(true);
        }
        previousScroll = scrollPosition;
    };

    /**
     * Retrieve content from list dataSource
     * @param  {Boolean} apply - Scope apply selector
     */
    this.loadData = function(apply) {
        if (apply) {
            $scope.$apply();
        }
        var widget = $scope.widget;
        this.nextPage(widget).then(function(callback) {
        	var data;
            if (callback && callback.dataset) {
                Object.keys(callback.dataset).forEach(function (datasetName) {
                    data = callback.dataset[datasetName];
                });
            } else {
                data = callback || [];
            }
            $scope.loadedAllData = data.length < widget.itemsPerPage;
            $scope.loadingData = false;
        }.bind(this));
    };
    
    this.nextPage = function(widget) {
        if (widget.dataSource) {
            return widget.dataSource.load(widget.getDataSourceFilter(), widget.dataSource.getNextPage(), widget.itemsPerPage);
        } else {
            return ZHPromise.when();
        }
    };
    
    this.searchPatient = function(widget){
        var search = angular.copy(widget.typeSearch);
        if(search){
            search = removerAcentos(search);
        }
        if(!widget.dataSourceFilter){
            widget.dataSourceFilter = [{
                name: "patientName",
                operator: "LIKE",
                value: "%"+search+"%"
            }];
        }else{
            var noDataFound = true;
            widget.dataSourceFilter.forEach(function(filter){
               if(filter.name === 'patientName'){
                   filter.value = "%"+search+"%";
                   noDataFound = false;
               } 
            });
            if(noDataFound){
                widget.dataSourceFilter.push({
                    name: "patientName",
                    operator: "LIKE",
                    value: "%"+search+"%"
                });
            }
        }
        widget.reload();
    };
    
    function removerAcentos(newStringComAcento) {
        var string = newStringComAcento;
        var mapaAcentosHex = {
            a: /[\xE0-\xE6]/g,
            A: /[\xC0-\xC6]/g,
            e: /[\xE8-\xEB]/g,
            E: /[\xC8-\xCB]/g,
            i: /[\xEC-\xEF]/g,
            I: /[\xCC-\xCF]/g,
            o: /[\xF2-\xF6]/g,
            O: /[\xD2-\xD6]/g,
            u: /[\xF9-\xFC]/g,
            U: /[\xD9-\xDC]/g,
            c: /\xE7/g,
            C: /\xC7/g,
            n: /\xF1/g,
            N: /\xD1/g
        };

        for (var letra in mapaAcentosHex) {
            var expressaoRegular = mapaAcentosHex[letra];
            string = string.replace(expressaoRegular, letra);
        }

        return string;
    }

    container.on("scroll", this.infiniteScroll.bind(this));

    this.deliverFood = function(widget, service, index) {
        var widget = $scope.widget;        
        var message = 'Deseja confirmar entrega?';
        ScreenService.confirmMessage(message, 'question', function() {
            eventAggregator.publish("sendDeliverFood", {'service': service,'widget': widget,'index':index});
        });
    };
    
    this.collectFood = function(widget, service, index) {
        var widget = $scope.widget;        
        var message = 'Deseja confirmar recolhimento?';
        ScreenService.confirmMessage(message, 'question', function() {
            eventAggregator.publish("sendCollectFood", {'service': service,'widget': widget, 'index':index});
        });
    };
}
